<?php
/**
 * Soubor CDOList.php obsahujici definici tridy CDOList.
 */

/**
 * Trida poskytujici metody pro pracovani se seznamem CDO
 */
class CDOList
{
    /** @var array $hidList seznam id hesel */
    private $hidList;

    /** @var CDO $cdo CDO instance vytvorena pro potreby CDOList */
    private $cdo;

    /**
     * CDOList constructor.
     * @param $hidList array seznam hid
     */
    public function __construct($hidList = array())
    {
        $this->hidList = $hidList;
        $this->objList = array();
    }

    /**
     * Sestaveni JOIN stringu pro query.
     * @param $data
     * @return string
     */
    protected function buildJoinPart($data)
    {
        $joinStr = '';
        foreach ($data as $t => $join) {
            if ($join['main']) {
                $joinStr .= ' LEFT JOIN ';
                $joinStr .= $t;
                $joinStr .= ' ON ' . implode('=', $join['main']);
            } else {
                $joinStr .= $t;
            }
            if ($join['child']) {
                foreach ($join['child'] as $ch) {
                    $part = $this->buildJoinPart($ch);
                    $joinStr .= $part ? '' . $part . '' : '';
                }
            }

        }

        return $joinStr;
    }

    /**
     * Parsing vysledku CDOList query.
     * @param $result
     * @return array
     */
    protected function resultPostProcess($result)
    {
        $parsed = array();
        foreach ($result as $row) {
            $id_dict = array();
            foreach ($row as $idx => $val) {
//                if (is_null($val)) continue;
                if (strpos($idx, 'heslo-') !== 0) {
                    $idx = 'heslo-' . $idx;
                }
                /*
                 * název každého pole se nám vrátí ve formátu :
                 * heslo-varianta-typ_hesla--id
                 * čili string za "--" je vlastní název sloupce,
                 * zbytek je seznam rodičů
                 */
                $tmp = explode('--', $idx);
                $parents = explode('-', $tmp[0]);
                $colname = $tmp[1];
                /*
                 * pokud máme sloupec "id", pak si ho uložíme pro danou posloupnost rodičů
                 * např. $id_dict['heslo-varianta-typ_hesla'] = id
                 */
                if ($colname == 'id') {
                    $id_dict[$tmp[0]] = $val;
                }
                $pref = &$parsed;
                foreach ($parents as $n => $p) {
                    /*
                     * pro každou položku v posloupnosti rodičů založíme strukturu
                     */
                    if (!isset($pref[$p])) {
                        $pref[$p] = array();
                    }
                    /*
                     * pro aktuální položku najdeme id v id_dict
                     * a posuneme se na další nadřazenou položku v seznamu
                     */
                    $q = implode('-', array_slice($parents, 0, $n + 1));
                    $idname = $p . '-' . $id_dict[$q];
                    if (!isset($pref[$p][$idname])) {
                        $pref[$p][$idname] = array();
                    }
                    $pref = &$pref[$p][$idname];
                }
                /*
                 * vložíme hodnotu na správné místo ve struktuře
                 */
                $pref[$colname] = $val;

            }
        }
        return $parsed;
    }

    /**
     * Sestaveni CDOList query.
     * @param array|bool $fields
     * @return array
     */
    protected function buildQuery($fields = False)
    {
        // vytvoří se dummy CDO objekt, abychom měli dostupnou strukturu hesla
        $this->cdo = new CDO(CDOin_SQL, 'temp');
        // vygenerování částí SQL dotazu z dummy CDO
        $sqlParts = $this->cdo->getSQL($fields);
        // vytvoření stringu s názvy sloupců
        $colString = implode(', ', $sqlParts->data['columns']);
        // vytvoření stringu pro JOIN
        $joinStr = $this->buildJoinPart($sqlParts->data['tables']);

        $orderBy = array(array('FIELD([heslo].[id], %sql)', implode(',', $this->hidList)));
        if (isset($fields['varianta'])) {
            $orderBy['varianty.sortorder'] = 'ASC';
        }
        if (isset($fields['vyznam'])) {
            $orderBy['vyznamy.sortorder'] = 'ASC';
            if (isset($fields['vyznam']['podhesli'])) {
                $orderBy['podhesli_vyznamu.id'] = 'ASC';
            }
        }
//        dibi::test('
//            SELECT ' . $colString . '
//            FROM ' . $joinStr . '
//            WHERE [heslo].[id] IN %in
//            ORDER BY %by
//        ', $this->hidList, $orderBy);
        $result = dibi::fetchAll('
            SELECT ' . $colString . '
            FROM ' . $joinStr . '
            WHERE [heslo].[id] IN %in
            ORDER BY %by
        ', $this->hidList, $orderBy);
        return $this->resultPostProcess($result);
    }

    public function tabExport($fields = False)
    {
        $data = $this->buildQuery($fields);
        $tf = $this->cdo->tabFormat($data['heslo'], $fields);
//        var_dump($data);
//        var_dump($tf);
//        $this->tabHtml($tf);
        return $tf;
    }

    private function formatRecursive($data, $prefix = '')
    {
        $output = array();
        if (is_array($data)) {
            foreach ($data as $idx => $d) {
                if ($prefix) {
                    $idx = '-' . $idx;
                }
                $output = array_merge($output, $this->formatRecursive($d, $prefix . $idx));
            }
        } else {
            $output[$prefix] = $data;
        }
        return $output;
    }

    public function tabHtml($data)
    {
        $rows = array();
        foreach ($data as $row) {
            $rows[] = $this->formatRecursive($row);
        }
        $headers = array_keys($rows[0]);
        $html = '<table id="tab_data">';
        $html .= '<tr>';
        foreach ($headers as $h) {
            $tmp = explode('-', $h);
            $html .= '<th>' . $tmp[count($tmp) - 1] . '</th>';
        }
        $html .= '</tr>';
        foreach ($rows as $row) {
            $cls = isset($row['heslo']) ? 'first_hrow' : '';
            $html .= '<tr class="' . $cls . '">';
            foreach ($headers as $h) {
                if (isset($row[$h])) {
                    $html .= '<td>' . $row[$h] . '</td>';
                } else {
                    $html .= '<td></td>';
                }
            }
            $html .= '</tr>';
        }
        $html .= '</table>';

        return $html;
    }

    public function tabCsv($data)
    {
        header('Content-Type: text/csv; charset=utf-8');
        header('Content-Disposition: attachment; filename=tabulka.csv');
        $output = fopen('php://output', 'w');
        $rows = array();
        foreach ($data as $row) {
            $rows[] = $this->formatRecursive($row);
        }
        $headers = array_keys($rows[0]);
        $csvRow = array();
        foreach ($headers as $h) {
            $tmp = explode('-', $h);
            $csvRow[] = $tmp[count($tmp) - 1];
        }
        fputcsv($output, $csvRow, ';');
        foreach ($rows as $row) {
            $csvRow = array();
            foreach ($headers as $h) {
                if (isset($row[$h])) {
                    $csvRow[] = strip_tags($row[$h]);
                } else {
                    $csvRow[] = '';
                }

            }
            fputcsv($output, $csvRow, ';');
        }
    }
}